/*
 *   Wormhole X-Treme Plugin for Bukkit
 *   Copyright (C) 2011  Ben Echols
 *                       Dean Bailey
 *
 *   This program is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package com.wormhole_xtreme.wormhole;

import java.util.logging.Level;

import org.bukkit.Location;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.entity.Player;
import org.bukkit.event.block.BlockBreakEvent;
import org.bukkit.event.block.BlockBurnEvent;
import org.bukkit.event.block.BlockDamageEvent;
import org.bukkit.event.block.BlockFromToEvent;
import org.bukkit.event.block.BlockIgniteEvent;
import org.bukkit.event.block.BlockListener;
import org.bukkit.event.block.BlockPhysicsEvent;

import com.wormhole_xtreme.wormhole.model.Stargate;
import com.wormhole_xtreme.wormhole.model.StargateManager;
import com.wormhole_xtreme.wormhole.permissions.WXPermissions;
import com.wormhole_xtreme.wormhole.permissions.WXPermissions.PermissionType;
import com.wormhole_xtreme.wormhole.utils.WorldUtils;

/**
 * WormholeXTreme Block Listener.
 * 
 * @author Ben Echols (Lologarithm)
 * @author Dean Bailey (alron)
 */
class WormholeXTremeBlockListener extends BlockListener
{
    /**
     * Handle block break.
     * 
     * @param player
     *            the player
     * @param stargate
     *            the stargate
     * @param block
     *            the block
     * @return true, if successful
     */
    private static boolean handleBlockBreak(final Player player, final Stargate stargate, final Block block)
    {
        final boolean allowed = WXPermissions.checkWXPermissions(player, stargate, PermissionType.DAMAGE);
        if (allowed)
        {
            if ( !WorldUtils.isSameBlock(stargate.getGateDialLeverBlock(), block))
            {
                if ((stargate.getGateDialSignBlock() != null) && WorldUtils.isSameBlock(stargate.getGateDialSignBlock(), block))
                {
                    player.sendMessage("Destroyed DHD Sign. You will be unable to change dialing target from this gate.");
                    player.sendMessage("You can rebuild it later.");
                    stargate.setGateDialSign(null);
                }
                else if (block.getTypeId() == (stargate.isGateCustom()
                    ? stargate.getGateCustomIrisMaterial().getId()
                    : stargate.getGateShape() != null
                        ? stargate.getGateShape().getShapeIrisMaterial().getId()
                        : 1))
                {
                    return true;
                }
                else
                {
                    if (stargate.isGateActive())
                    {
                        stargate.setGateActive(false);
                        stargate.fillGateInterior(0);
                    }
                    if (stargate.isGateLightsActive())
                    {
                        stargate.lightStargate(false);
                        stargate.stopActivationTimer();
                        StargateManager.removeActivatedStargate(player);
                    }
                    stargate.resetTeleportSign();
                    stargate.setupGateSign(false);
                    if ( !stargate.getGateIrisDeactivationCode().equals(""))
                    {
                        stargate.setupIrisLever(false);
                    }
                    if (stargate.isGateRedstonePowered())
                    {
                        stargate.setupRedstone(false);
                    }
                    StargateManager.removeStargate(stargate);
                    player.sendMessage("Stargate Destroyed: " + stargate.getGateName());
                }
            }
            else
            {
                player.sendMessage("Destroyed DHD. You will be unable to dial out from this gate.");
                player.sendMessage("You can rebuild it later.");
            }
            return false;
        }
        else
        {
            if (player != null)
            {
                WormholeXTreme.getThisPlugin().prettyLog(Level.FINE, false, "Player: " + player.getName() + " denied block destroy on: " + stargate.getGateName());
            }
        }
        return true;
    }

    /* (non-Javadoc)
     * @see org.bukkit.event.block.BlockListener#onBlockBreak(org.bukkit.event.block.BlockBreakEvent)
     */
    @Override
    public void onBlockBreak(final BlockBreakEvent event)
    {
        if ( !event.isCancelled())
        {
            final Block block = event.getBlock();
            final Stargate stargate = StargateManager.getGateFromBlock(block);
            final Player player = event.getPlayer();
            if ((stargate != null) && handleBlockBreak(player, stargate, block))
            {
                event.setCancelled(true);
            }
        }
    }

    /* (non-Javadoc)
     * @see org.bukkit.event.block.BlockListener#onBlockBurn(org.bukkit.event.block.BlockBurnEvent)
     */
    @Override
    public void onBlockBurn(final BlockBurnEvent event)
    {
        if ( !event.isCancelled())
        {
            final Location current = event.getBlock().getLocation();
            final Stargate closest = StargateManager.findClosestStargate(current);
            //TODO This is bad, very bad for performance!
            if ((closest != null) && (closest.isGateActive() || closest.isGateRecentlyActive()) && ((closest.isGateCustom()
                ? closest.getGateCustomPortalMaterial()
                : closest.getGateShape() != null
                    ? closest.getGateShape().getShapePortalMaterial()
                    : Material.STATIONARY_WATER) == Material.STATIONARY_LAVA))
            {
                final double blockDistanceSquared = StargateManager.distanceSquaredToClosestGateBlock(current, closest);
                if (((blockDistanceSquared <= (closest.isGateCustom()
                    ? closest.getGateCustomWooshDepthSquared()
                    : closest.getGateShape() != null
                        ? closest.getGateShape().getShapeWooshDepthSquared()
                        : 0)) && ((closest.isGateCustom()
                    ? closest.getGateCustomWooshDepth()
                    : closest.getGateShape() != null
                        ? closest.getGateShape().getShapeWooshDepth()
                        : 0) != 0)) || (blockDistanceSquared <= 25))
                {
                    WormholeXTreme.getThisPlugin().prettyLog(Level.FINE, false, "Blocked Gate: \"" + closest.getGateName() + "\" Proximity Block Burn Distance Squared: \"" + blockDistanceSquared + "\"");
                    event.setCancelled(true);
                }
            }
        }
    }

    /* (non-Javadoc)
     * @see org.bukkit.event.block.BlockListener#onBlockDamage(org.bukkit.event.block.BlockDamageEvent)
     */
    @Override
    public void onBlockDamage(final BlockDamageEvent event)
    {
        if ( !event.isCancelled())
        {
            final Stargate stargate = StargateManager.getGateFromBlock(event.getBlock());
            final Player player = event.getPlayer();
            if ((stargate != null) && (player != null) && !WXPermissions.checkWXPermissions(player, stargate, PermissionType.DAMAGE))
            {
                event.setCancelled(true);
                WormholeXTreme.getThisPlugin().prettyLog(Level.FINE, false, "Player: " + player.getName() + " denied damage on: " + stargate.getGateName());
            }
        }
    }

    /* (non-Javadoc)
     * @see org.bukkit.event.block.BlockListener#onBlockFlow(org.bukkit.event.block.BlockFromToEvent)
     */
    @Override
    public void onBlockFromTo(final BlockFromToEvent event)
    {
        if ( !event.isCancelled())
        {
            if (StargateManager.isBlockInGate(event.getToBlock()) || StargateManager.isBlockInGate(event.getBlock()))
            {
                event.setCancelled(true);
            }
        }
    }

    /* (non-Javadoc)
     * @see org.bukkit.event.block.BlockListener#onBlockIgnite(org.bukkit.event.block.BlockIgniteEvent)
     */
    @Override
    public void onBlockIgnite(final BlockIgniteEvent event)
    {
        if ( !event.isCancelled())
        {
            final Location current = event.getBlock().getLocation();
            final Stargate closest = StargateManager.findClosestStargate(current);
            if ((closest != null) && (closest.isGateActive() || closest.isGateRecentlyActive()) && ((closest.isGateCustom()
                ? closest.getGateCustomPortalMaterial()
                : closest.getGateShape() != null
                    ? closest.getGateShape().getShapePortalMaterial()
                    : Material.STATIONARY_WATER) == Material.STATIONARY_LAVA))
            {
                final double blockDistanceSquared = StargateManager.distanceSquaredToClosestGateBlock(current, closest);
                if (((blockDistanceSquared <= (closest.isGateCustom()
                    ? closest.getGateCustomWooshDepthSquared()
                    : closest.getGateShape().getShapeWooshDepthSquared())) && ((closest.isGateCustom()
                    ? closest.getGateCustomWooshDepth()
                    : closest.getGateShape() != null
                        ? closest.getGateShape().getShapeWooshDepth()
                        : 0) != 0)) || (blockDistanceSquared <= 25))
                {
                    WormholeXTreme.getThisPlugin().prettyLog(Level.FINE, false, "Blocked Gate: \"" + closest.getGateName() + "\" Block Type: \"" + event.getBlock().getType().toString() + "\" Proximity Block Ignite: \"" + event.getCause().toString() + "\" Distance Squared: \"" + blockDistanceSquared + "\"");
                    event.setCancelled(true);
                }
            }
        }
    }

    /* (non-Javadoc)
     * @see org.bukkit.event.block.BlockListener#onBlockPhysics(org.bukkit.event.block.BlockPhysicsEvent)
     */
    @Override
    public void onBlockPhysics(final BlockPhysicsEvent event)
    {
        if ( !event.isCancelled())
        {
            final Block block = event.getBlock();
            if (StargateManager.isBlockInGate(block) && (block.getTypeId() != 55))
            {
                event.setCancelled(true);
            }
        }
    }
}
